(function(){

let stripes = ["r", "g", "b"];

let generateStripes = function(x, y, c){
	let width = Math.floor(board.x / 3);
	let genType = function(t, count, exclude, xoffset){
		for(let i = 0; i < count; i++){
			let x_ = xoffset + 3 * Math.floor(Math.random() * width);
        	let y_ = Math.floor(Math.random() * board.y);
			let tile = board.at(x_, y_);
			let valid = true;
			if(tile == null) valid = false;
			if(x == x_ && y == y_) valid = false;
			if(valid && tile.mine != "") valid = false;
			if(valid && (Math.random() > tile.probability)) valid = false;
			if(valid){
				for(let d of exclude){
					if(board.posEqual(x + d[0], x_, y + d[1], y_)){
						valid = false;
						break;
					}
				}
			}
			if(valid){
				tile.mine = t;
			}else{
				i--;
			}
		}
	};
	genType("r", activeMineCount("R"), shape.small, 0);
	genType("g", activeMineCount("G"), shape.small, 1);
	genType("b", activeMineCount("B"), shape.small, 2);
	genType("R", activeMineCount("RX"), shape.large, 0);
	genType("G", activeMineCount("GX"), shape.large, 1);
	genType("B", activeMineCount("BX"), shape.large, 2);
	for(let i = 0; i < board.x; i++){
		for(let j = 0; j < board.y; j++){
			tile = board.at(i, j);
			if(tile == null) continue;
			tile.surroundingMines = accumulateSurrounding(i, j, "mine");
		}
	}
	if("postGenFunction" in activeSettings){
		activeSettings["postGenFunction"](tile, x, y);
	}
	if("spawn" in activeSettings){
		Entities.spawn(activeSettings.spawn, x, y);
	}
};

registerDay(55,
{
	"title" : "Stripes?",
	"desc" : "No please don't leave.<br>"
		+ "Trust me it's fine.<br>"
		+ "Wait, please stay, let me explain.<br>"
		+ "This is not the same mess that the 2018 Stripes was. I am not going to do this again for today.<br><br>"
		+ "Instead it's the mines that generate in stripes. The leftmost column of the board can only contain red mines."
		+ " The second one can only contain green mines, the third one only blue mines, the fourth one only red mines and so it repeats."
		+ " Don't worry about counting stripes, placing a flag will automatically place the flag of the corresponding stripe.<br><br>"
		+ "<center><img src=\"desc/50_Colorcharge_sufgsfzzgv.png\"/></center><br>"
		+ " This leads to some unique logic for today. Lets take this example:<br><br>"
		+ "<center><img src=\"desc/50_Example1.png\"/></center><br>"
		+ "Here we can observe the highlighted blue 1, the corresponding mine has to be on one of the tiles above."
		+ " And due to the stripes we know that only C is capable of generating a blue mine, while A could only be red and B could only be green."
		+ " Thus A and B are safe while C is the blue mine.<br><br>"
		+ "<center><img src=\"desc/50_Example2.png\"/></center><br>"
		+ "Both the green 2 and the yellow 1 are already satisfied, however both still have three closed tiles next to them and with colorcharge there is the risk of three mines cancelling out."
		+ " Yet today this is only possible for the yellow 1 where the three tiles above all belong to different stripes."
		+ " For the green 2 this isn't possible as all those 3 tiles are in the same stripe and thus can't contain one mine of each type. This means that the three tiles with green dots are safe."
		+ " The yellow dots we can not determine as of now, they may be safe but they may also all be mines.<br><br>"
		+ "<center><img src=\"desc/50_Blue3.png\"/></center><br>"
		+ " Imagine you randomly click (for whatever reason) and a reveal a blue 3 all the surrounding tiles are still closed."
		+ " Usually you really wouldn't want to guess the surrounding tiles. Yet here you wouldn't even need to guess as that situation is completely solvable."
		+ " There is a blue stripe either to the left or right. You can figure out which one it is just by placing flags."
		+ " Now all 3 surrounding tiles in that stripe are mines and as there can't be a fourth blue mine there is no way for a green and red mine to cancel out with another blue mine."
		+ " And thus the other 5 tiles surrounding the blue 3 are safe.<br><br>" 
		+ " You may have noticed the dark red numbers on the sides going in opposite directions. Unlike the blue edges used for cylindrical boards, this indicates a Möbius strip."
		+ " Like in the cylinder where the right side connects to the left side so you could imagine the edges as glued together."
		+ " However, now the edges are glued together in opposite directions so the top right is connected to the bottom left and bottom right is connected to the top left.",
	"mines" : {
		"R" : 30,
		"G" : 30,
		"B" : 30,
	},
	"grayMines" : false,
	"display" : "colorcharge",
	"decrementing" : false,
	"x" : 21,
	"y" : 21,
	"xwrap" : 2,
	"ywrap" : 0,
	"board" : "6U/79xVAABEw+ROf6ABgAes2T4I",
	"customFlagFunction" : function(tile, x, y){
		if(tile.flag == ""){
			setFlag(x, y, stripes[x % 3]);
		}else{
			setFlag(x, y, "");
		}
	},
	"customDigFunction" : function(tile, x, y){
		let start = false;
		if(gamestate == "start"){
			runGame();
			start = true;
			generateStripes(x, y);
		}
		tile.open = true;
		if((tile.surroundingMines == "") && (tile.mine == "")) openSurrounding(x, y);
		Entities.onClick(x, y, start ? Entities.events.startGame : Entities.events.digOrChord);
	},
});

})();
