(function(){

let d61mode = "edit";

let customSections = {
	main: document.getElementById("Day61Desc"),
	play: document.getElementById("secd61-play"),
	edit: document.getElementById("secd61-edit"),
	json: document.getElementById("secd61-json"),
};

let softInitBoard = function(){
	if(!("x" in editSettings)) editSettings.x = 20;
	if(!("y" in editSettings)) editSettings.y = 20;
	if(!("board" in editSettings)) editSettings.board = "P8";
	board.x = editSettings.x;
	board.y = editSettings.y;
	board.xwrap = ("xwrap" in editSettings) ? editSettings.xwrap : 0;
	board.ywrap = ("ywrap" in editSettings) ? editSettings.ywrap : 0;
	setCanvasSize();
	board.tiles = [];
	let rawData = decompress(editSettings.board, board.x, board.y);
	for(let x = 0; x < board.x; x++){
		board.tiles[x] = [];
		for(let y = 0; y < board.y; y++){
			if(rawData[x][y] == 0){
				board.tiles[x][y] = null;
			}else{
				board.tiles[x][y] = {
					probability: (rawData[x][y] - 1) / 254,
					mine: "",
					surroundingMines: "",
					surroundingFlags: "",
					open: false,
					flag: "",
					effect: "",
				};
			}
		}
	}
	Entities.clear();
	setTimer(0, counter.timer);
	renderGame();
};

let playtest = function(){
	checkDirtyBoard();
	checkDirtyDesc();
	let checkGen = false;
	switch(d61mode){
		case "edit":
			editToJson();
			jsonToPlay();
			checkGen = true;
			break;
		case "json":
			if(!jsonToEdit()) return;
			jsonToPlay();
			checkGen = true;
			break;
	}
	d61mode = "play";
	revealElement(customSections.play);
	hideElement(customSections.edit);
	hideElement(customSections.json);
	days[61] = playSettings;
	currentDay = 61;
	activeSettings = playSettings;
	setTimer(0, counter.timer);
	gamestate = "start";
	toggleIndicator();
	let mineCount = 0;
	for(let i of ["r", "rx", "g", "gx", "b", "bx"]) {
		let element = counter["section_" + i];
		let n = activeMineCount(i);
		mineCount += n;
		if(n){
			element.className = "";
		}else{
			element.className = "hide";
		}
	}
	setDesc();
	initBoard();
	if(checkGen && !(("fixedPattern" in activeSettings) && activeSettings.fixedPattern)){
		let sum = 0;
		for(let x = 0; x < board.x; x++){
			for(let y = 0; y < board.y; y++){
				let t = board.at(x, y);
				if(t == null) continue;
				sum += Math.min(1, 3 * t.probability);
			}
		}
		sum -= shape.small.length + shape.large.length + 1;
		if(sum < mineCount){
			openWarning("<b>These settings may not be able to generate!</b><br>"
				+ "This setting has failed some sanity checks for it will be able to generate the necessary amount of mines on the board."
				+ " This does not necessarily mean that this setting can't generate a board. This may be a false positive."
				+ " Invalid settings may crash the game, in which case you may have to force close this browser tab.<br><br>"
				+ "Continue at your own risk!");
		}
	}
};

let editBasic = function(){
	checkDirtyBoard();
	checkDirtyDesc();
	switch(d61mode){
		case "json":
			if(!jsonToEdit()) return;
			break;
	}
	activeSettings = editSettings;
	d61mode = "edit";
	gamestate = "edit";
	toggleIndicator();
	hideElement(customSections.play);
	revealElement(customSections.edit);
	hideElement(customSections.json);
	softInitBoard();
	// Board Size
	xSize.value = editSettings.x;
	ySize.value = editSettings.y;
	// Mines
	mineList.toDisplay();
	// Topology
	let tpMode = (("xwrap" in editSettings) ? ("" + editSettings.xwrap) : "0") + (("ywrap" in editSettings) ? ("" + editSettings.ywrap) : "0");
	topologyRadio.setValue(tpMode);
	// Neighbourhood
	smallCount.binding = null;
	largeCount.binding = null;
	if("shape" in editSettings){
		if("small" in editSettings.shape) smallCount.binding = editSettings.shape.small;
		if("large" in editSettings.shape) largeCount.binding = editSettings.shape.large;
	}
	smallCount.draw();
	largeCount.draw();
	// Display Style
	displayStyleRadio.setValue(("display" in editSettings) ? editSettings.display : "");
	grayMinesCheckbox.checked = ("grayMines" in editSettings) ? editSettings.grayMines : true;
	decrementCheckbox.checked = ("decrementing" in editSettings) ? editSettings.decrementing : false;
	fixedPatternCheckbox.checked = ("fixedPattern" in editSettings) ? editSettings.fixedPattern : false;
	updateFixedPatternDisplay();
	// Animals
	for(let a in animals) animals[a].checked = false;
	if(("spawn" in editSettings) && Array.isArray(editSettings.spawn)){
		for(let s of editSettings.spawn){
			console.log(s);
			if(s in animals) animals[s].checked = true;
		}
	}
	chordingCheckbox.checked = ("disableChord" in editSettings) ? !editSettings.disableChord : true;
	// Title Author Desc
	titleText.value = editSettings.title;
	authorText.value = editSettings.author;
	descriptionText.value = editSettings.desc;

};

let relProbSec = document.getElementById("probability-selector");
let tileSelSec = document.getElementById("tile-selector");
let updateFixedPatternDisplay = function(){
	let fixedPattern = fixedPatternCheckbox.checked;
	if(fixedPattern) {
		hideElement(relProbSec);
		revealElement(tileSelSec);
	}else{
		revealElement(relProbSec);
		hideElement(tileSelSec);
	}
	renderGame();
};

let editJson = function(){
	checkDirtyBoard();
	checkDirtyDesc();
	switch(d61mode){
		case "edit":
			editToJson();
			break;
	}
	d61mode = "json";
	gamestate = "json";
	toggleIndicator();
	hideElement(customSections.play);
	hideElement(customSections.edit);
	revealElement(customSections.json);
	renderGame();
};

let setUpload = null;

let setDesc = function(){
	if(options.offsetToOne){
		pageSections.titleDay.innerText = "Day 24 - " + activeSettings.title;
	}else{
		pageSections.titleDay.innerText = "Minesveeper 61 - " + activeSettings.title;
	}
	let desc = document.getElementById("desc61-text");
	desc.innerHTML = "";
	let addBr = false;
	for(let line of activeSettings.desc.split("\n")){
		if(addBr) desc.appendChild(document.createElement("br"));
		let s = document.createElement("span");
		s.innerText = line;
		desc.appendChild(s);
		addBr = true;
	}
	document.getElementById("desc61-author").innerText = activeSettings.author;
	document.getElementById("d61-colorcharge-chart").className = (activeSettings.display == "colorcharge") ? "" : "hide";
	verified = false;
	setUpload();
};


document.getElementById("db61-play").onclick = playtest;
document.getElementById("db61-edit").onclick = editBasic;
document.getElementById("db61-json").onclick = editJson;

let editSettings = {
	"title" : "Customization",
	"desc" : "No Description Provided",
	"author" : "No Name Provided",
	"mines" : {
		"R" : 60,
	},
	"grayMines" : true,
	"display" : "default",
	"shape" : {
		"small" : [[-1, -1], [-1, 0], [-1, 1], [0, -1], [0, 1], [1, -1], [1, 0], [1, 1]],
		"large" : [
			[-2, -2], [-2, -1], [-2, 0], [-2, 1], [-2, 2],
			[-1, -2], [-1, -1], [-1, 0], [-1, 1], [-1, 2],
			[0, -2], [0, -1], [0, 1], [0, 2],
			[1, -2], [1, -1], [1, 0], [1, 1], [1, 2],
			[2, -2], [2, -1], [2, 0], [2, 1], [2, 2],
		],
	},
	"decrementing" : false,
	"x" : 20,
	"y" : 20,
	"board" : "P8",
};
let jsonSettings = null;
let playSettings = editSettings;

let jsonToEdit = function(){
	var data = validateJSON();
	if(data === false){
		return false;
	}else{
		editSettings = data;
		jsonSettings = jsonBox.value;
		return true;
	}
};

let editToJson = function(){
	mineList.toSetting();
	jsonSettings = JSON.stringify(editSettings, null, 1);
	// Insert into textbox
	jsonBox.value = jsonSettings;
	validateResult.innerText = "";
};

let jsonToPlay = function(){
	verified = false;
	playSettings = JSON.parse(jsonSettings);
	// Resolve constants and copyFrom instructions
	let jConst = ("constants" in playSettings) ? playSettings["constants"] : {};
	let resolveRec = function(obj, path){
		if(Array.isArray(obj)){
			for(let k in obj){
				path.push(k);
				let replace = resolveRec(obj[k], path);
				if(replace != null) obj[k] = replace;
				path.pop();
			}
		}else{
			switch(typeof(obj)){
				case "object":
					for(let k in obj){
						path.push(k);
						let replace = resolveRec(obj[k], path);
						if(replace != null) obj[k] = replace;
						path.pop();
					}
					break;
				case "string":
					if(obj in jConst) return jConst[obj];
					if(obj.startsWith("copyFrom:")){
						let source = +obj.substr(9);
						if(source in days){
							let i = days[source];
							for(let j of path){
								if(!(j in i)) return null;
								i = i[j];
							}
							return i;
						}
					}
					break;
			}
		}
		return null;
	};
	resolveRec(playSettings, []);
	// Set Active Settings and start game
};

let startSetting = null;

day61Fnct = {
	open: () => {
		setTimer(0, counter.timer);
		if(options.offsetToOne){
			pageSections.titleDay.innerHTML = "Day 24 - Customization";
		}else{
			pageSections.titleDay.innerHTML = "Minesveeper 61 - Customization";
		}
		revealElement(customSections.main);
		console.log("Start Custom");
		switch(d61mode){
			case "play":
				playtest();
				break;
			case "edit":
				editBasic();
				break;
			case "json":
				editJson();
				break;
		}
	},
	close: () => {
		hideElement(customSections.main);
		checkDirtyBoard();
		console.log("Close Custom");
	},
	jsonRender: () => {
		context.font = "14px Arial";
		context.textAlign = "left";
		context.fillText("Board display is disabled,", 10, 16);
		context.fillText("while in the Json Editor!", 10, 34);
	},
	editLeft : (x, y) => {
		let fp = ("fixedPattern" in editSettings) && editSettings.fixedPattern;
		paint(x, y, fp ? tileSelector.selection : propabilitySelector.selection, fp);
		renderGame();
	},
	editRight : (x, y) => {
		let fp = ("fixedPattern" in editSettings) && editSettings.fixedPattern;
		paint(x, y, fp ? 2 : 0, fp);
		renderGame();
	},
	verify: () => {
		if(verified) return;
		verified = true;
		setUpload();
	},
	setCustom: (data) => {
		let loadMetadata = document.getElementById("loadMetadata");
		let customGame = loadMetadata.getAttribute("custom") + "";
		try {
			startSetting = JSON.parse(doReplace(data));
			uploadText = data;
			uploaded[data] = customGame;
		} catch(ex) {
			return;
		}
	},
	startCustom: () => {
		if(startSetting == null) return;
		const loadMetadata = document.getElementById("loadMetadata");
		const flags = loadMetadata.getAttribute("customFlags");
		let message = "";
		if((flags & 2) == 2){
			// Warn about inappropriate settings
			message = "<b>Inappropriate Content</b><br>"
			 	+ "This custom setting was flagged as inappropriate content."
				+ " While it did not get completely deleted, it was marked as not appropriate for everyone.<br><br>"
				+ " Continue at your own risk!";
		}
		if((!options.userContent) && (flags & 1) == 0) {
			// Warn about user submitted content
			if(message.length > 0) message += "<br><br>";
			message += "<b>Warning: Player Submitted Content</b><br>"
			message += "You are playing a setting submitted by another player."
				+ " I can make no promises about this setting"
				+ " This setting should also be taken as an endorsement of the title or description."
				+ " If you believe that this setting violates the rules for custom settings, please report it either via Discord or the E-Mail address on the About page.<br><br>"
				+ " Continue at your own risk!"
				+ " If you do not wish to see this message again, you can disable it in the settings.";
		}
		if(message.length > 0){
			openChoice(message, startIntoCustomGame, ()=>{});
		}else{
			startIntoCustomGame();
		}
	},
	isFixed: () => { return ("fixedPattern" in editSettings) ? editSettings.fixedPattern : false; },
};

let parseRadio = (name) => {
	let obj = {
		elements: document.getElementsByName(name),
		onValueChanged: null,
		setValue: null,
		getValue: null,
	};
	let getVal = () => {
		for(let e of obj.elements){
			if(e.checked) return e.value;
		}
		return "";
	};
	obj.getValue = getVal;
	let change = () => {
		if(obj.onValueChanged != null){
			obj.onValueChanged(getVal());
		}
	};
	obj.setValue = (v) => {
		for(let e of obj.elements){
			e.checked = v == e.value;
		}
	};
	for(let e of obj.elements){
		e.onchange = change;
	}
	return obj;
};

let parseSelectionCanvas = (name, textures) => {
	let obj = {
		canvas: document.getElementById(name),
		context: null,
		draw: null,
		selection: 0,
		textures: textures,
	};
	obj.context = obj.canvas.getContext("2d");
	obj.canvas.height = 16;
	obj.canvas.width = 16 * textures.length;
	obj.draw = () => {
		for(let i = 0; i < obj.textures.length; i++){
			obj.context.drawImage(obj.textures[i], 16 * i, 0);	
		}
		obj.context.drawImage(Textures.edit.select, obj.selection * 16, 0);
	};
	obj.draw();
	obj.canvas.oncontextmenu = () => false;
	obj.canvas.addEventListener('mouseup', function(event) {
		let i = event.pageX - obj.canvas.getBoundingClientRect().left - 2 - window.pageXOffset;
		obj.selection = Math.floor(i / 16);
		obj.draw();
	}, false);
	return obj;
};

// Board Size

let boardDirty = false;

let xSize = document.getElementById("d61-s-x");
let ySize = document.getElementById("d61-s-y");
let adjustModeRadio = parseRadio("adjust");
let adjustModeDirection = parseRadio("adjustdir");

let parseIntInput = (i, def) => {
	n = parseInt(i.value);
	if(isNaN(n)){
		n = def;
		i.value = def;
	}
	return n;
};

document.getElementById("applyShape").onclick = ()=> {
	let replica = (t) => {
		if(t == null) return null;
		return {
			probability: t.probability,
			mine: "",
			surroundingMines: "",
			surroundingFlags: "",
			open: false,
			flag: "",
			effect: "",
		};
	};
	let newX = parseIntInput(xSize, 20);
	let newY = parseIntInput(ySize, 20);
	if(board.x == newX && board.y == newY) return;
	editSettings.x = newX;
	editSettings.y = newY;
	let newTiles = [];
	let adjMode = adjustModeRadio.getValue();
	let adjDir = adjustModeDirection.getValue();
	let basePosX = 0;
	let basePosY = 0;
	switch(adjDir){
		case "ul":
			basePosX = newX - board.x;
			basePosY = newY - board.y;
			break;
		case "dr":
			basePosX = 0;
			basePosY = 0;
			break;
		case "ce":
			basePosX = Math.floor((newX - board.x) / 2);
			basePosY = Math.floor((newY - board.y) / 2);
			break;
	}
	for(let x = 0; x < newX; x++){
		newTiles[x] = [];
		for(let y = 0; y < newY; y++){
			let s = null;
			let sourceX = x - basePosX;
			let sourceY = y - basePosY;
			switch(adjMode){
				case "sample":
					// Doesn't care about the adjust direction
					s = board.at(Math.floor(x / newX * board.x), Math.floor(y / newY * board.y));
					break;
				case "extrude":
					if(sourceX < 0) sourceX = 0;
					if(sourceY < 0) sourceY = 0;
					if(sourceX >= board.x) sourceX = board.x - 1;
					if(sourceY >= board.y) sourceY = board.y - 1;
					s = board.at(sourceX, sourceY);
					break;
				case "repeat":
					sourceX %= board.x;
					if(sourceX < 0) sourceX += board.x;
					sourceY %= board.y;
					if(sourceY < 0) sourceY += board.y;
					s = board.at(sourceX, sourceY);
					break;
			}
			newTiles[x][y] = replica(s);
		}
	}
	board.x = newX;
	board.y = newY;
	board.tiles = newTiles;
	boardDirty = true;
	setCanvasSize();
	renderGame();
};


// Edit Board
let brushRadio = parseRadio("brush");
let probabilityOptions = [0, 0, 1, 32, 64, 85, 127, 169, 190, 222, 254];

let propabilitySelector = parseSelectionCanvas("prob_canvas", [
	Textures.edit.oob,
	Textures.background.number[0],
	Textures.background.fractions.epsilon,
	Textures.background.fractions.frac_1_8,
	Textures.background.fractions.frac_1_4,
	Textures.background.fractions.frac_1_3,
	Textures.background.fractions.frac_1_2,
	Textures.background.fractions.frac_2_3,
	Textures.background.fractions.frac_3_4,
	Textures.background.fractions.frac_7_8,
	Textures.background.number[1],
]);
propabilitySelector.selection = propabilitySelector.textures.length - 1;
propabilitySelector.draw();

let tileSelectOptions = [0, 0, 254, 64, 85, 127, 169, 190, 222];

let tileSelector = parseSelectionCanvas("tile_select_canvas", [
	Textures.edit.oob,
	Textures.background.starter,
	Textures.background.clear,
	Textures.background.mine.R,
	Textures.background.mine.RX,
	Textures.background.mine.G,
	Textures.background.mine.GX,
	Textures.background.mine.B,
	Textures.background.mine.BX,
]);
tileSelector.selection = 1;
tileSelector.draw();

let paint = function(x, y, p, fixedPattern){
	let size = parseInt(brushRadio.getValue());
	let v = fixedPattern ? tileSelectOptions[p] : probabilityOptions[p];
	x -= Math.floor(size / 2);
	y -= Math.floor(size / 2);
	for(let i = 0; i < size; i++){
		for(let j = 0; j < size; j++){
			let x_ = x + i;
			let y_ = y + j;
			if(x_ >= 0 && y_ >= 0 && x_ < board.x && y_ < board.y){
				if(p == 0){
					board.tiles[x_][y_] = null;
				}else{
					board.tiles[x_][y_] = {
						probability: v / 254,
						mine: "",
						surroundingMines: "",
						surroundingFlags: "",
						open: false,
						flag: "",
						effect: "",
					};
				}
			}
		}
	}
	boardDirty = true;
	renderGame();
}

let checkDirtyBoard = () => {
	if(!boardDirty) return;
	boardDirty = false;
	let data = [];
	for(let x = 0; x < board.x; x++) {
		for(let y = 0; y < board.y; y++) {
			let t = board.at(x, y);
			if(t == null){
				data.push(0);
			}else{
				data.push(1 + Math.floor(254.01 * t.probability));
			}
		}
	}
	let b64Board = compress(data);
	//console.log(b64Board);
	editSettings.board = b64Board;
};

// Mines

let mineList = {
	elements : ["R", "RX", "G", "GX", "B", "BX"],
	toSetting : () => {
		for(let e of mineList.elements){
			let n = parseIntInput(e[1], 0);
			if(n > 0){
				editSettings.mines[e[0]] = n;
			}else if(e[0] in editSettings.mines){
				delete editSettings.mines[e[0]];
			}
		}
	},
	toDisplay : () => {
		for(let e of mineList.elements){
			let n = 0;
			if(e[0] in editSettings.mines) n = parseInt(editSettings.mines[e[0]]);
			if(isNaN(n)) n = 0;
			e[1].value = n;
		}
	},
};

for(let i = 0; i < 6; i++){
	mineList.elements[i] = [mineList.elements[i], document.getElementById("d61-m-" + mineList.elements[i])];
	mineList.elements[i][1].onchange = mineList.toSetting;
}

mineList.toDisplay();

// Topology

let topologyRadio = parseRadio("topology");
topologyRadio.onValueChanged = (val) => {
	let xwrap = parseInt(val[0]);
	let ywrap = parseInt(val[1]);
	board.xwrap = xwrap;
	board.ywrap = ywrap;
	editSettings["xwrap"] = xwrap;
	editSettings["ywrap"] = ywrap;
	setCanvasSize();
	renderGame();
};

// Neighbourhood
let countSelector = parseSelectionCanvas("count_canvas", [
	Textures.edit.count1,
	Textures.edit.countR,
	Textures.edit.countG,
	Textures.edit.countB,
]);

let parseNeighbourhoodCanvas = (name, isLarge) => {
	let obj = {
		canvas: document.getElementById(name),
		context: null,
		draw: null,
		binding: null,
	};
	obj.context = obj.canvas.getContext("2d");
	obj.draw = () => {
		if(!Array.isArray(obj.binding)){
			for(let x = 0; x < 9; x++){
				for(let y = 0; y < 9; y++){
					if(x == 4 && y == 4){
						obj.context.drawImage(Textures.background.questionmark, 16 * x, 16 * y);	
					}else{
						obj.context.drawImage(Textures.background.clear, 16 * x, 16 * y);	
					}
				}
			}
			return;
		}
		for(let x = 0; x < 9; x++){
			for(let y = 0; y < 9; y++){
				if(x == 4 && y == 4){
					obj.context.drawImage(Textures.edit.eye, 16 * x, 16 * y);	
				}else{
					let tex = Textures.background.clear;
					for(let s of obj.binding){
						if(Array.isArray(s) && s.length > 1 && s[0] == x - 4 && s[1] == y - 4){
							if(s.length == 2){
								tex = Textures.edit.count1;
							}else{
								switch("" + s[2]){
									case "r":
									case "R":
										tex = Textures.edit.countR;
										break;
									case "g":
									case "G":
										tex = Textures.edit.countG;
										break;
									case "b":
									case "B":
										tex = Textures.edit.countB;
										break;
									default:
										tex = Textures.background.questionmark;
								}
							}
						}
					}
					obj.context.drawImage(tex, 16 * x, 16 * y);	
				}
			}
		}
	};
	obj.canvas.oncontextmenu = () => false;
	obj.canvas.addEventListener('mouseup', function(event) {
		if(!Array.isArray(obj.binding)) return;
		let i = event.pageX - obj.canvas.getBoundingClientRect().left - 2 - window.pageXOffset;
		let j = event.pageY - obj.canvas.getBoundingClientRect().top - 2 - window.pageYOffset;
		let x = Math.floor(i / 16) - 4;
		let y = Math.floor(j / 16) - 4;
		if(x == 0 && y == 0) return;
		let newTile = [x, y];
		switch(countSelector.selection){
			case 1:
				newTile.push(isLarge ? "R" : "r");
				break;
			case 2:
				newTile.push(isLarge ? "G" : "g");
				break;
			case 3:
				newTile.push(isLarge ? "B" : "b");
				break;
		}
		let found = false;
		for(let n = 0; n < obj.binding.length; n++){
			let s = obj.binding[n];
			if(Array.isArray(s) && s.length > 1 && s[0] == x && s[1] == y){
				if(s.length == 2 && newTile.length == 2){
					obj.binding[n] = obj.binding[obj.binding.length - 1];
					obj.binding.pop();
				}else if(s.length == 3 && newTile.length == 3 && ("" + s[2] == "" + newTile[2])){
					obj.binding[n] = obj.binding[obj.binding.length - 1];
					obj.binding.pop();
				}else{
					obj.binding[n] = newTile;
				}
				found = true;
				break;
			}
		}
		if(!found){
			obj.binding.push(newTile);
		}
		obj.draw();
	}, false);
	return obj;	
};

let smallCount = parseNeighbourhoodCanvas("small_mine_canvas", false);
let largeCount = parseNeighbourhoodCanvas("large_mine_canvas", true);

// Display Style
let displayStyleRadio = parseRadio("dispstyle");
displayStyleRadio.onValueChanged = (val) => {
	editSettings.display = val;
};

let decrementCheckbox = document.getElementById("d61-dec");
decrementCheckbox.onchange = () => {
	editSettings.decrementing = decrementCheckbox.checked;
};

let grayMinesCheckbox = document.getElementById("d61-graymines");
grayMinesCheckbox.onchange = () => {
	editSettings.grayMines = grayMinesCheckbox.checked;
};

let fixedPatternCheckbox = document.getElementById("d61-fixed-pattern");
fixedPatternCheckbox.onchange = () => {
	if(fixedPatternCheckbox.checked){
		editSettings.fixedPattern = true;
	}else{
		delete editSettings.fixedPattern;
	}
	updateFixedPatternDisplay();
};

// Animals
let animals = {};

let animalListChanged = () => {
	let list = [];
	if(animals.sheep.checked) list.push("sheep");
	if(animals.rat.checked) {
		list.push("cheese");
		list.push("rat");
	}
	if(animals.dog.checked) {
		list.push("ball");
		list.push("dog");
	}
	if(animals.horse.checked) list.push("horse");
	editSettings.spawn = list;
};

for(let name of ["sheep", "rat", "dog", "horse"]){
	animals[name] = document.getElementById("d61-" + name);
	animals[name].onchange = animalListChanged;
}

let chordingCheckbox = document.getElementById("d61-allowChord");
chordingCheckbox.onchange = () => {
	if(chordingCheckbox.checked){
		delete editSettings.disableChord;
	}else{
		editSettings.disableChord = true;
	}
};

// Title Author Desc
let descDirty = false;
let descChange = () => { descDirty = true; };

let titleText = document.getElementById("d61-title");
titleText.oninput = descChange;

let authorText = document.getElementById("d61-author");
authorText.oninput = descChange;

let descriptionText = document.getElementById("d61-description");
descriptionText.oninput = descChange;

let checkDirtyDesc = () => {
	if(!descDirty) return;
	descDirty = false;
	editSettings.title = titleText.value;
	editSettings.author = authorText.value;
	editSettings.desc = descriptionText.value;
};

let jsonBox = document.getElementById("d61-json");
let validateResult = document.getElementById("json-syntax-result");

let validateJSON = () => {
	try {
		let res = JSON.parse(jsonBox.value);
		if(typeof(res) != "object"){
			validateResult.innerText = "Settings must be an Object";
			return false;
		}
		validateResult.innerText = "Valid Json";
		return res;
	} catch(ex) {
		validateResult.innerText = "" + ex;
		openWarning("<b>Invalid Json</b><br>Make sure to provide a <b><a href=\"https://www.json.org\">Valid JSON Object</a></b>, before you can return to the regular editor or play your setting!<br><br> " + ex);
		return false;
	}
};

document.getElementById("checkJson").onclick = validateJSON;

editToJson();

let uploadText = "";
let verified = false;
let uploaded = {};
let userAgrees = false;

let uploadSec = {
	verify: document.getElementById("gen-to-share"),
	upload: document.getElementById("upload-your-setting"),
	share: document.getElementById("use-link-to-share"),
	link: document.getElementById("custom-link"),
	uploadButton: document.getElementById("share-button"),
	uploadFailed: document.getElementById("upload-failed"),
	agree: document.getElementById("d61-rules"),
};


uploadSec.agree.checked = false;
uploadSec.agree.onchange = function(){
	userAgrees = uploadSec.agree.checked;
	if(userAgrees){
		uploadSec.uploadButton.className = "greenbutton";
	}else{
		uploadSec.uploadButton.className = "redbutton";
	}
};

let replaceMap = {
	"\"": "_", "_": "\"",
	"A": "!", "!": "A",
	"E": "?", "?": "E",
	"a": "$", "$": "a",
	"e": "&", "&": "e",
	"O": "@", "@": "O",
	"o": ";", ";": "o",
	"\\": "%", "%": "\\"
};

// Unfortunately the hosting provider I use checks any web request for SQL like commands
// and blocks access to the site if something like this is found, since it is quite easy
// to get false possitive with this I am going to obfuscate all uploaded data, so even if
// you write DROP TABLE in the description everything should still work fine.
// Yes, there has been an elaborate amount of E-Mails going back and forth of me requesting
// this to be turned off as I can sanitize the data myself. However, there fucking support
// is to fucking incompetent to actually do anything about it and just keeps redirecting
// me to other even more fucking incompetent people who don't understand the problem and
// having me explain the same fucking shit all over again. FUCK!
let doReplace = i => i.replace(/A|E|a|e|O|o|\!|\?|\$|\&|@|;|\\|%|_|\"/g, i => replaceMap[i]);

let baseLink = "heptaveegesimal.com/2023/advent-calendar?c=";

setUpload = function(){
	uploadText = doReplace(JSON.stringify(editSettings));
	hideElement(uploadSec.verify);
	hideElement(uploadSec.upload);
	hideElement(uploadSec.share);
	if(uploadText in uploaded){
		verified = true;
		revealElement(uploadSec.share)
		let link = baseLink + uploaded[uploadText];
		uploadSec.link.href = "https:\\\\" + link;
		uploadSec.link.innerText = link;
	}else if(verified){
		revealElement(uploadSec.upload);
		uploadSec.agree.checked = false;
		uploadSec.uploadButton.className = "redbutton";
		uploadSec.uploadFailed.innerHTML = "";
	}else{
		revealElement(uploadSec.verify);
	}
};

let requestJSON = function(param, callBack, onError){
	let url = "https://heptaveegesimal.com/2023/advent-calendar/upload.php";
	let xhr = new XMLHttpRequest();
	xhr.open('POST', url, true);
	xhr.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
	if(callBack){
		xhr.onload = function () {
			let result = null;
			try{
				result = JSON.parse(this.responseText);
			}catch(e){
				console.log(this.responseText);
				console.log(e);
				if(onError) onError(true);
			}
			// Make sure that we only catch errors during parsing
			if(result !== null){
				callBack(result);
			}
		};
	}
	if(onError){
		xhr.onerror = function() {
			onError(false);
		};
	}
	let paramStr = "";
	for(key in param){
		//console.log(encodeURI(param[key].toString()));
		paramStr += ((paramStr.length == 0) ? "" : "&") + encodeURIComponent(key) + "=" + encodeURIComponent(param[key].toString());
	}
	xhr.send(paramStr);
};

uploadSec.uploadButton.onclick = function(){
	if(!uploadSec.agree.checked){
		uploadSec.uploadFailed.innerHTML = "<br><br>You cannot upload your custom settings unless you read and agree to the rules and data policy of uploading custom settings!";
		return;
	}
	requestJSON({data: uploadText},
	(json)=>{
		console.log(json);
		switch(json.result){
			case 1:
			case 2:
				uploaded[uploadText] = json.link;
				setUpload();
				break;
			case -1:
				uploadSec.uploadFailed.innerHTML = "<br><br>Request rejected by the server:<br>" + json.text;
				break;
			default:
				uploadSec.uploadFailed.innerHTML = "<br><br>Unknown server response code:" + json.result;
				break;
		}
	},
	(b) => {
		uploadSec.uploadFailed.innerHTML = "<br><br>Upload Failed (No or invalid server response)!<br>Maybe try again later!"
		+ " You can save the Json representation of your setting in the meantime, so that you don't loose any progress.<br><br>"
		+ " If this problem persists, please report this issue to me either via E-Mail or Discord!";
	});

};

let startIntoCustomGame = function(){
	jsonSettings = JSON.stringify(startSetting, null, 1);
	jsonBox.value = jsonSettings;
	d61mode = "json";
	playtest();
	document.getElementById("dayButton_24").onclick();
};

document.getElementById("d61-opt").className = "";

})();
