(function(){

Textures.background.zen = {
	"flowerBlue" : Textures.fnct.load("zen/Flower_Blue.png"),
	"flowerCyan" : Textures.fnct.load("zen/Flower_Cyan.png"),
	"flowerGreen" : Textures.fnct.load("zen/Flower_Green.png"),
	"flowerYellow" : Textures.fnct.load("zen/Flower_Yellow.png"),
	"flowerRed" : Textures.fnct.load("zen/Flower_Red.png"),
	"flowerMagenta" : Textures.fnct.load("zen/Flower_Magenta.png"),
	"fountain" : Textures.fnct.load("zen/Fountain.png"),
	"carrots" : Textures.fnct.load("zen/Carrots.png"),
	"carrotsEaten" : Textures.fnct.load("zen/Carrots_Eaten.png"),
	"campfire" : Textures.fnct.load("zen/Campfire.png"),
	"stone3" : Textures.fnct.load("zen/Stone_3.png"),
};

let dCarrot = [[-5, -5], [-4, -4], [-5, 5], [-4, 4], [5, -5], [4, -4], [5, 5], [4, 4]];

let tabRow = function(name, img, desc){
	return "<tr class=\"bluetab\"><td class=\"bluetab\">" + name + "<td class=\"bluetab\"><img src=\"images/zen/" + img + ".png\"><td class=\"bluetab\">" + desc;
}

registerDay(67, 
{
	"title" : "A New Garden",
	"desc" : "Hey, we haven't had any colorcharge days in the extension so far!<br>"
		+ "Let's change that and let's add a bit more decoration to the colorful board.<br>"
		+ "If you have played the 2018 advent calendar, you may remember the Zen Garden mechanic."
		+ " Various decorative features replace safe tiles and all them have their own unique meanings."
		+ " This time it works in a similar manner, although with some new decorative tiles."
		+ " So what are they and what do they mean?<br><br>"
		+ "<center><table class=\"bluetab\">"
		+ tabRow("Red Flower", "Flower_Red", "This flower represents any pure red number, with no mines canceling out")
		+ tabRow("Yellow Flower", "Flower_Yellow", "This flower represents any pure yellow number, with no mines canceling out")
		+ tabRow("Green Flower", "Flower_Green", "This flower represents any pure green number, with no mines canceling out")
		+ tabRow("Cyan Flower", "Flower_Cyan", "This flower represents any pure cyan number, with no mines canceling out")
		+ tabRow("Blue Flower", "Flower_Blue", "This flower represents any pure blue number, with no mines canceling out")
		+ tabRow("Magenta Flower", "Flower_Magenta", "This flower represents any pure magenta number, with no mines canceling out")
		+ tabRow("Fountain", "Fountain", "The 7 by 7 area centered around this fountain, does not contain a single tile in which colorcharges cancel out")
		+ tabRow("Carrot Field", "Carrots", "The 8 tiles which are 4 or 5 tiles away diagonally are all safe")
		+ tabRow("Campfire", "Campfire", "For the 8 tiles surrounding the campfire, out of each pair of opposite tiles one contains a mine while the other one is safe")
		+ tabRow("Stone 3", "Stone_3", "This tile is surrounded by exactly 3 mines and it could not have been replaced by any of the above features")
		+ "</table></center><br>"
		+ "Of course here is also the obligatory colorcharge chart:<br><br>"
		+ "<center><img src=\"desc/46_Colorcharge.png\" style=\"max-width:90%\"/></center><br>",
	"mines" : {
		"R" : 33,
		"G" : 33,
		"B" : 33,
	},
	"grayMines" : false,
	"display" : "colorcharge",
	"shape" : "default",
	"decrementing" : false,
	"x" : 28,
	"y" : 26,
	"xwrap" : 1,
	"ywrap" : 1,
	"board": "8/6LACACEvyA6ZCKMjNGRNNAIAOjAEjQnsCLIyl3pF21FtGX9GRW9Iw2ou2wGJKFkomenn9MVpe2f0xo",
	"zenDensity" : {
		"flowerBlue" : 0.1,
		"flowerCyan" : 0.1,
		"flowerGreen" : 0.1,
		"flowerYellow" : 0.1,
		"flowerRed" : 0.1,
		"flowerMagenta" : 0.1,
		"fountain" : 0.06,
		"carrots" : 0.1,
		"campfire" : 0.75,
		"stone3" : 0.1,
	},
	"postGenFunction" : function(_tile, _x, _y){
		for(let x = 0; x < board.x; x++){
			for(let y = 0; y < board.y; y++){
				let t = board.at(x, y);
				if(t == null) continue;
				t.rgb = [0, 0, 0, 0];
				for(let c of t.surroundingMines.toLowerCase()){
					switch(c){
						case "r":
							t.rgb[0]++;
							break;
						case "g":
							t.rgb[1]++;
							break;
						case "b":
							t.rgb[2]++;
							break;
					}
				}
				let m = Math.min(t.rgb[0], t.rgb[1], t.rgb[2]);
				t.rgb[3] = m;
				t.rgb[0] -= m;
				t.rgb[1] -= m;
				t.rgb[2] -= m;
			}
		}
		for(let x = 0; x < board.x; x++){
			for(let y = 0; y < board.y; y++){
				let t = board.at(x, y);
				if(t == null) continue;
				t.zen = [];
				if(x == _x && y == _y) continue;
				if(t.rgb[3] == 0){
					if(t.rgb[0] > 0 && t.rgb[1] == 0 && t.rgb[2] == 0) t.zen.push("flowerRed");
					if(t.rgb[0] == 0 && t.rgb[1] > 0 && t.rgb[2] == 0) t.zen.push("flowerGreen");
					if(t.rgb[0] == 0 && t.rgb[1] == 0 && t.rgb[2] > 0) t.zen.push("flowerBlue");
					if(t.rgb[0] > 0 && t.rgb[1] == t.rgb[0] && t.rgb[2] == 0) t.zen.push("flowerYellow");
					if(t.rgb[0] == 0 && t.rgb[1] > 0 && t.rgb[2] == t.rgb[1]) t.zen.push("flowerCyan");
					if(t.rgb[0] == t.rgb[2] && t.rgb[1] == 0 && t.rgb[2] > 0) t.zen.push("flowerMagenta");
				}
				let fountain = true;
				for(let dx = -3; dx <= 3; dx++){
					for(let dy = -3; dy <= 3; dy++){
						let t = board.at(x + dx, y + dy);
						if(t == null) continue;
						if(t.rgb[3] > 0){
							fountain = false;
							dx = 10;
							dy = 10;
						}
					}
				}
				if(fountain) t.zen.push("fountain");
				let carrot = true;
				for(let d of dCarrot){
					let t = board.at(x + d[0], y + d[1]);
					if(t == null) continue;
					if(t.mine != "") {
						carrot = false;
					}
				}
				if(carrot) t.zen.push("carrots");
				let campfire = true;
				for(let d of [[1, 0], [-1, 1], [0, -1], [1, 1]]){
					let t1 = board.at(x + d[0], y + d[1]);
					let t2 = board.at(x - d[0], y - d[1]);
					if((t1 == null) || (t2 == null)) {
						campfire = false;
						break;
					}
					if(t1.mine == "" && t2.mine == "") campfire = false;
					if(t1.mine != "" && t2.mine != "") campfire = false;
					if(!campfire) break;
				}
				if(campfire) t.zen.push("campfire");
				if(t.zen.length == 0 && t.surroundingMines.length == 3) t.zen.push("stone3");
			}

		}
		let zd = activeSettings["zenDensity"];
		let density = (s) => { return (s in zd) ? zd[s] : 0; };
		for(let x = 0; x < board.x; x++){
			for(let y = 0; y < board.y; y++){
				let t = board.at(x, y);
				if(t == null) continue;
				if(t.zen.length == 0) continue;
				let sum = 0;
				for(let s of t.zen) sum += density(s);
				let r = Math.random();
				if(r >= sum) continue;
				if(sum > 1) r *= sum;
				let p = 0;
				for(let s of t.zen){
					r -= density(s);
					if(r < 0){
						p = s;
						break;
					}
				}
				if(p == 0) continue;
				t.surroundingMines = ["background", "zen", p];
			}
		}
	},
});

})();
